@@ -6,13 +6,24 @@ import stat
 import psutil
 import argparse
 import json
+import platform
 from datetime import datetime
-from humetools import NotImplementedAction, printerr, pprinterr
+from humetools import (
+    NotImplementedAction, printerr, pprinterr, valueOrDefault, envOrDefault,
+    is_valid_hostname
+)
 
-__version__ = '1.2.11'
+
+__version__ = '1.2.13'
 
 
 class Hume():
+    LEVELS = ['info', 'ok', 'warning', 'error', 'critical', 'debug', ]
+    DEFAULT_LEVEL = 'info'
+    NO_TAGS = []
+    NO_TASKID = ''
+    RECVTIMEOUT = 1000
+
     def __init__(self, args):
         self.config = {'url': 'tcp://127.0.0.1:198'}
 
@@ -25,18 +36,35 @@ class Hume():
         # in such a way that the code can grow organically
         # and be coder-assistive
         self.reqObj = {}
+        # Stores hostname
+        self.reqObj['hostname'] = args.hostname
         # To store information related to how hume was executed
         self.reqObj['process'] = {}
         # Hume-specific information
         self.reqObj['hume'] = {}
-        self.reqObj['hume']['level'] = args.level
-        self.reqObj['hume']['msg'] = args.msg
-        self.reqObj['hume']['tags'] = args.tags
-        self.reqObj['hume']['task'] = args.task
-        self.reqObj['hume']['humecmd'] = args.humecmd
+        # Mandatory
         self.reqObj['hume']['timestamp'] = self.get_timestamp()
-        if self.args.append_pstree:
-            self.reqObj['process']['tree'] = self.get_pstree()
+        # Make sure to set a default value
+        self.reqObj['hume']['level'] = valueOrDefault(args,
+                                                      'level',
+                                                      Hume.DEFAULT_LEVEL)
+        self.reqObj['hume']['tags'] = valueOrDefault(args,
+                                                     'tags',
+                                                     Hume.NO_TAGS)
+        self.reqObj['hume']['task'] = valueOrDefault(args,
+                                                     'task',
+                                                     Hume.NO_TASKID)
+        self.reqObj['hume']['msg'] = valueOrDefault(args, 'msg', '')
+        # Very optional ones:
+        try:
+            self.reqObj['hume']['humecmd'] = args.humecmd
+        except AttributeError:
+            pass
+        try:
+            if self.args.append_pstree:
+                self.reqObj['process']['tree'] = self.get_pstree()
+        except AttributeError:
+            pass
 
         ln = self.get_lineno()
         if ln is not None:
@@ -69,7 +97,8 @@ class Hume():
         # self.reqObj to a special structure:
         # {'payload': ENCRYPTED_ASCII_ARMORED_CONTENT,
         #  'encrypted': True}
-        # or something like that
+        # or something like that. Also, probably kant would
+        # be useful in this context...
         if encrypt_to is None:
             HUME = self.reqObj
         else:
@@ -95,7 +124,9 @@ class Hume():
             sys.exit(4)
         poller = zmq.Poller()
         poller.register(sock, zmq.POLLIN)
-        if poller.poll(self.args.recvtimeout):
+        if poller.poll(valueOrDefault(self.args,
+                                      'recvtimeout',
+                                      Hume.RECVTIMEOUT)):
             msg = sock.recv_string().strip()
         else:
             print('Timeout sending hume')
@@ -138,11 +169,12 @@ class Hume():
 
 def run():
     parser = argparse.ArgumentParser()
+    parser.add_argument('--version',
+                        action='version',
+                        version='HumeClient v{} by Buanzo'.format(__version__))
     parser.add_argument("-L", "--level",
-                        choices=['ok', 'info',
-                                 'warning', 'error',
-                                 'critical', 'debug'],
-                        default="info",
+                        choices=Hume.LEVELS,
+                        default=Hume.DEFAULT_LEVEL,
                         help="Level of update to send, defaults to 'info'")
     parser.add_argument("-c", "--hume-cmd",
                         choices=['counter-start',
@@ -155,33 +187,48 @@ def run():
                         help="[OPTIONAL] Command to attach to the update.")
     parser.add_argument("-t", "--task",
                         required=False,
-                        default='',
-                        help="[OPTIONAL] Task name, for example BACKUPTASK.")
+                        default=envOrDefault('HUME_TASKNAME', ''),
+                        help='''[OPTIONAL] Task name, for example BACKUPTASK.
+Takes precedente over HUME_TASKNAME envvar.''')
     parser.add_argument('-a', '--append-pstree',
                         action='store_true',
                         help="Append process calling tree")
     parser.add_argument('-T', '--tags',
-                        type=lambda arg: arg.split(','),
                         action='append',
-                        help="Comma-separated list of tags")
+                        default=[envOrDefault('HUME_TAGS', '')],
+                        help='''Comma-separated list of tags. HUME_TAGS
+envvar contents are appended.''')
     parser.add_argument('-e', '--encrypt-to',
                         default=None,
                         action=NotImplementedAction,
                         dest='encrypt_to',
                         help="[OPTIONAL] Encrypt to this gpg pubkey id")
     parser.add_argument('--recv-timeout',
-                        default=1000,
+                        default=int(envOrDefault('HUME_RECVTIMEOUT', 1000)),
                         type=int,
                         dest='recvtimeout',
                         help='''Time to wait for humed reply to hume message.
-Default 1000ms / 1 second.''')
+Default 1000ms / 1 second. Takes precedence over HUME_RECVTIMEOUT envvar.''')
+    parser.add_argument('--hostname',
+                        default=platform.node(),
+                        dest='hostname',
+                        help='''[OPTIONAL] Sets hostname to use in hume
+message. Defaults to detected hostname "{}"'''.format(platform.node()))
     parser.add_argument('msg',
                         help="[REQUIRED] Message to include with this update")
     args = parser.parse_args()
+    
+    if not is_valid_hostname(args.hostname):
+        printerr('Hostname is not valid. Ignoring hume.')
+        sys.exit(1)
 
     # Allows for multiple --tags tag1,tag2 --tags tag3,tag4 to be a simple list
+    fulltags = []
     if args.tags is not None:
-        args.tags = [item for sublist in args.tags for item in sublist]
+        for item in args.tags:
+            if len(item) > 0:
+                fulltags.extend(item.split(','))
+        args.tags = fulltags
     else:
         args.tags = []
 
