@@ -1,6 +1,7 @@
 #!/usr/bin/python3
 import logging
 from logging import getLogger
+from hume import __version__
 import sys
 import zmq
 import json
@@ -14,7 +15,7 @@ from logging.handlers import SysLogHandler
 from pid.decorator import pidfile
 from queue import Queue
 from threading import Thread
-from humetools import printerr, pprinterr
+from humetools import printerr, pprinterr, is_valid_hostname
 # The Confuse library is awesome.
 import confuse
 
@@ -59,6 +60,7 @@ config_template = {  # TODO: add debug. check confuse.Bool()
         'webhook_error': confuse.String(),
         'webhook_critical': confuse.String(),
         'webhook_debug': confuse.String(),
+        'template': confuse.String(),  # TODO: use ['template'].as_filename()
     },
 }
 
@@ -237,6 +239,7 @@ class Humed():
         except Exception as ex:
             return(False)  # FIX: malformed json at this stage? mmm
         hume = humepkt['hume']
+        sender_host = humepkt['hostname']
         if self.debug:
             pprinterr(hume)
         level = hume['level']
@@ -249,11 +252,12 @@ class Humed():
             tagstr = ','.join(tags)
         # Make sure to read:
         # https://api.slack.com/reference/surfaces/formatting
-        m = "[{ts}] - {level} {task}: '{msg}' {tagstr}".format(level=level,
-                                                               msg=msg,
-                                                               task=task,
-                                                               ts=ts,
-                                                               tagstr=tagstr)
+        m = "{} [{ts}] - {level} {task}: '{msg}' {tagstr}".format(sender_host,
+                                                                  level=level,
+                                                                  msg=msg,
+                                                                  task=task,
+                                                                  ts=ts,
+                                                                  tagstr=tagstr)
         # https://api.slack.com/reference/surfaces/formatting#escaping
         m = m.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')
         # Remember, text becomes a fallback if 'blocks' are in use:
@@ -293,6 +297,7 @@ class Humed():
         except Exception as ex:
             return(False)  # FIX: malformed json at this stage? mmm
         hume = humepkt['hume']
+        sender_host = humepkt['hostname']
         if 'process' in humepkt.keys():  # This data is optional in hume (-a)
             process = humepkt['process']
         else:
@@ -310,6 +315,7 @@ class Humed():
         hostname = socket.getfqdn()
         # extra field for logstash message
         extra = {
+            'hostname': sender_host,
             'tags': tags,
             'task': task,
             'humelevel': level,
@@ -364,6 +370,7 @@ class Humed():
         except Exception as ex:
             return(False)  # FIX: malformed json at this stage? mmm
         hume = humepkt['hume']
+        sender_host = humepkt['hostname']
 
         # Optional data
         if 'process' in humepkt.keys():  # This data is optional in hume (-a)
@@ -388,12 +395,11 @@ class Humed():
         hostname = socket.getfqdn()
 
         # We dont have the 'extra' field for syslog, in contrast to logstash
-        msg = '[{}-{}-{}] TAGS=[{}] HUMECMD={} MSG={}'.format(timestamp,
-                                                              task,
-                                                              humelevel,
-                                                              tags,
-                                                              humecmd,
-                                                              msg)
+        msg = '{} {} {} [{}] {} | TAGS={}'.format(sender_host,
+                                                  task,
+                                                  humelevel,
+                                                  msg,
+                                                  tags)
         if process is not None:
             msg = '{} PROC={}'.format(msg,
                                       json.dumps(extra['process']))
@@ -426,6 +432,18 @@ class Humed():
         else:
             return(True)
 
+    def is_valid_hume(self,hume):
+        # This function checks incoming hume structure
+        # and values.
+        # Returns: True or False
+        if 'hostname' in hume.keys():
+            if not is_valid_hostname(hume['hostname']):
+                return(False)
+        else:  # hostname MUST exist and be valid
+            return(False)
+        pprinterr(hume)
+        return(True)  # FIX: Change to False once all cases are considered
+
     def run(self):
         # Humed main loop
         sock = zmq.Context().socket(zmq.REP)
@@ -452,13 +470,19 @@ class Humed():
                 printerr('CTRL-C called, exiting now')
                 sys.exit(255)
             else:
+                # TODO: validate hume HERE and provide response accordingly
+                # CLient MAY timeout before this happens so this SHOULD
+                # NOT affect be a deal breaker
                 sock.send_string('OK')
-                rowid = self.add_transfer(hume)
-                self.queue.put(('work'))
-                if self.debug:
-                    printerr(rowid)
-        # TODO: 2c - log errors and rowids
-        # TODO: deal with exits/breaks
+                if self.is_valid_hume(hume):
+                    rowid = self.add_transfer(hume)  # TODO: verify ret
+                    if self.debug:
+                        printerr(rowid)
+                    self.queue.put(('work'))
+                else:
+                    if self.debug:
+                        printerr('Received hume is not valid:')
+                        pprinterr(hume)
 
 
 @pidfile()
@@ -470,6 +494,9 @@ def main():
                         default=False,
                         action='store_true',
                         help='Enables debug messages')
+    parser.add_argument('--version',
+                        action='version',
+                        version='HumeDaemon v{} by Buanzo'.format(__version__))
     args = parser.parse_args()
     config.set_args(args)
     config.debug = args.debug
